==> commerce_findock-1.0.0-beta1.tar.gz: commerce_findock/src/PluginForm/OffsiteRedirect/FinDockForm.php <==
<?php

namespace Drupal\commerce_findock\PluginForm\OffsiteRedirect;

use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Component\Serialization\Json;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ServerException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Payment redirect form for FinDock.
 */
class FinDockForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Salesforce auth plugin manager.
   *
   * @var \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface
   */
  protected $salesForceAuthPluginManager;

  /**
   * FinDockForm constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface $salesforce_auth_plugin_manager
   *   The salesforce auth plugin manager.
   */
  public function __construct(Client $http_client, ModuleHandlerInterface $module_handler, SalesforceAuthProviderPluginManagerInterface $salesforce_auth_plugin_manager) {
    $this->httpClient = $http_client;
    $this->moduleHandler = $module_handler;
    $this->salesForceAuthPluginManager = $salesforce_auth_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('http_client'), $container->get('module_handler'), $container->get('plugin.manager.salesforce.auth_providers'));
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();

    $order = $payment->getOrder();
    $profile = $order->getBillingProfile();

    // @todo Make these settings configurable.
    $data = [
      'SuccessURL' => $form['#return_url'],
      'FailureURL' => $form['#cancel_url'],
      'Origin' => $payment->getPaymentGateway()->label(),
      'Payer' => [
        'AccountUpdate' => 'Enrich',
        'ContactUpdate' => 'Enrich',
        'AllowDeduplication' => TRUE,
        'PrimaryRelation' => 'Contact',
      ],
      'Payment' => [
        'Amount' => $payment->getAmount()->getNumber(),
        'CurrencyISOCode' => $payment->getAmount()->getCurrencyCode(),
      ],
      'PaymentMethod' => [
        'Name' => $configuration['payment_method'],
      ],
      'SourceConnector' => [
        'Name' => $configuration['source_connector'],
      ],
    ];

    if ($profile->hasField('address') && !$profile->get('address')->isEmpty()) {
      $data['Payer'] += [
        'Contact' => [
          'MailingStreet' => $profile->get('address')->address_line1,
          'MailingCity' => $profile->get('address')->locality,
          'MailingPostalCode' => $profile->get('address')->postal_code,
        ],
        'Account' => [
          'BillingStreet' => $profile->get('address')->address_line1,
          'BillingCity' => $profile->get('address')->locality,
          'BillingPostalCode' => $profile->get('address')->postal_code,
        ],
      ];
    }

    $this->moduleHandler->alter('commerce_findock_data', $data, $order, $payment);

    $event = NULL;
    if ($this->moduleHandler->moduleExists('past')) {
      $event = past_event_create('commerce_findock', 'payment_data', 'Payment #' . $order->id());
      $event->addArgument('data', json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
      $event->addArgument('order', $order);
      $event->addArgument('payment', $payment);
    }

    try {

      $token = $this->salesForceAuthPluginManager->getToken() ?? $this->salesForceAuthPluginManager->refreshToken();
      if (empty($token)) {
        throw new \Exception("Could not retrieve Auth Token. Is there a default salesforce auth provider set up?");
      }

      $params = [
        'json' => $data,
        'headers' => [
          'Authorization' => 'Bearer ' . $token->getAccessToken(),
          'verbose' => TRUE,
          'Content-Type' => 'application/json',
        ],
      ];

      $start = microtime(TRUE);

      try {
        $response = $this->httpClient->post($configuration['url'], $params);
      }
      catch (ClientException $e) {
        // On 401 'Invalid Session ID or Unauthorized', refresh the token and try again.
        // Reference: salesforce/src/Rest/RestClient.php:194
        if ($e->getCode() === 401) {
          if ($event) {
            $event->addArgument('authentication', 'Invalid Session ID caught. Refreshing Authentication Token now and try to post again.');
          }
          $token = $this->salesForceAuthPluginManager->refreshToken();
          $params['headers']['Authorization'] = 'Bearer ' . $token->getAccessToken();

          // Be more lenient with the default 30 second timeout,
          // since the token has just been refreshed.
          $params['timeout'] = 45;
          $response = $this->httpClient->post($configuration['url'], $params);
        }
        else {
          throw $e;
        }
      }

      $time = round((microtime(TRUE) - $start) * 1000, 2);
      if ($event) {
        $event->addArgument('time', $time);
      }

      $response_data = Json::decode((string) $response->getBody());
      if ($event) {
        $event->addArgument('response_data', $response_data);
      }
      if ($response_data['IsSuccess']) {

        if (isset($response_data['Payment']['InstallmentId'])) {
          $order->setData('findock_installment_id', $response_data['Payment']['InstallmentId']);
        }
        if (isset($response_data['Recurring']['RecurringId'])) {
          $order->setData('findock_recurring_id', $response_data['Recurring']['RecurringId']);
        }

        // If there is no redirect URL then the configured payment method
        // does not need additional processing. Create the payment and continue.
        if (!isset($response_data['RedirectURL'])) {

          if (empty($response_data['IsSuccess'])) {
            throw new \Exception($response_data['Error']['error_message'], $response_data['ResponseCode']);
          }

          // Save the payment and directly redirect to the next step.
          $payment->setState('authorization');
          $payment->setRemoteState('pending');
          $payment->setRemoteId($order->getData('findock_installment_id'));
          $payment->save();

          if (\Drupal::moduleHandler()->moduleExists('past')) {
            past_event_save('commerce_findock', 'installment_data', 'Installment data #' . $order->id() . '(Installment #' . $order->getData('findock_installment_id') . ')', ['data' => $response_data, 'payment' => $payment]);
          }

          /** @var \Drupal\commerce_checkout\Entity\CheckoutFlowInterface $checkout_flow */
          $checkout_flow = $order->get('checkout_flow')->entity;
          $checkout_flow_plugin = $checkout_flow->getPlugin();

          $next_step_id = $checkout_flow_plugin->getNextStepId($order->get('checkout_step')->value);
          $checkout_flow_plugin->redirectToStep($next_step_id);
        }
        else {
          $redirect_url = $response_data['RedirectURL'];
          $order->save();
        }
      }
      else {
        if ($event) {
          $event->setSeverity(RfcLogLevel::ERROR);
        }
        throw new \Exception($response_data['Error']['error_message'], $response_data['ResponseCode']);
      }
    }
    catch (NeedsRedirectException $e) {
      throw $e;
    }
    catch (ServerException $e) {
      if ($event) {
        $event->addException($e);
        $event->addArgument('response', (string) $e->getResponse()->getBody());
      }
      throw new PaymentGatewayException(t('Failed to initialize payment: @error', ['@error' => $e->getMessage()]), $e->getCode(), $e);
    }
    catch (\Exception $e) {
      if ($event) {
        $event->addException($e);
      }
      throw new PaymentGatewayException(t('Failed to initialize payment: @error', ['@error' => $e->getMessage()]), $e->getCode(), $e);
    }
    finally {
      if ($event) {
        $event->save();
      }
    }

    if (!empty($redirect_url)) {
      $form = $this->buildRedirectForm($form, $form_state, $redirect_url, []);
    }
    return $form;
  }

}

==> commerce_findock-2.0.0-beta1.tar.gz: commerce_findock/src/PluginForm/OffsiteRedirect/FinDockForm.php <==
<?php

namespace Drupal\commerce_findock\PluginForm\OffsiteRedirect;

use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Component\Serialization\Json;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\salesforce\Rest\RestException;
use Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Payment redirect form for FinDock.
 */
class FinDockForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Salesforce auth plugin manager.
   *
   * @var \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface
   */
  protected $salesForceAuthPluginManager;

  /**
   * FinDockForm constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface $salesforce_auth_plugin_manager
   *   The salesforce auth plugin manager.
   */
  public function __construct(Client $http_client, ModuleHandlerInterface $module_handler, SalesforceAuthProviderPluginManagerInterface $salesforce_auth_plugin_manager) {
    $this->httpClient = $http_client;
    $this->moduleHandler = $module_handler;
    $this->salesForceAuthPluginManager = $salesforce_auth_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('http_client'), $container->get('module_handler'), $container->get('plugin.manager.salesforce.auth_providers'));
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();

    $order = $payment->getOrder();
    $profile = $order->getBillingProfile();

    // @todo Make these settings configurable.
    $data = [
      'SuccessURL' => $form['#return_url'],
      'FailureURL' => $form['#cancel_url'],
      'Origin' => $payment->getPaymentGateway()->label(),
      'Payer' => [],
      'OneTime' => [
        'Amount' => $payment->getAmount()->getNumber(),
        'CurrencyISOCode' => $payment->getAmount()->getCurrencyCode(),
      ],
      'PaymentMethod' => [
        'Name' => $configuration['payment_method'],
        'Processor' => $configuration['processor'] ?: NULL,
        'Target' => $configuration['findock_target'] ?: NULL,
      ],
      'Settings' => [
        'SourceConnector' => $configuration['source_connector'],
      ]
    ];

    if ($profile->hasField('address') && !$profile->get('address')->isEmpty()) {
      $data['Payer'] += [
        'Contact' => [
          'SalesforceFields' => [
            'FirstName' => $profile->get('address')->given_name,
            'LastName' => $profile->get('address')->family_name,
            'MailingStreet' => $profile->get('address')->address_line1,
            'MailingCity' => $profile->get('address')->locality,
            'MailingPostalCode' => $profile->get('address')->postal_code,
            'MailingCountry' => $profile->get('address')->country_code,
          ],
        ],
        'Account' => [
          'SalesforceFields' => [
            'BillingStreet' => $profile->get('address')->address_line1,
            'BillingCity' => $profile->get('address')->locality,
            'BillingPostalCode' => $profile->get('address')->postal_code,
            'BillingCountry' => $profile->get('address')->country_code,
          ],
        ],
      ];
    }

    $this->moduleHandler->alter('commerce_findock_data', $data, $order, $payment);

    $event = NULL;
    if ($this->moduleHandler->moduleExists('past')) {
      $event = past_event_create('commerce_findock', 'payment_data', 'Payment #' . $order->id());
      $event->addArgument('data', json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
      $event->addArgument('order', $order);
      $event->addArgument('payment', $payment);
    }

    try {

      $token = $this->salesForceAuthPluginManager->getToken() ?? $this->salesForceAuthPluginManager->refreshToken();
      if (empty($token)) {
        throw new \Exception("Could not retrieve Auth Token. Is there a default salesforce auth provider set up?");
      }

      $start = microtime(TRUE);

      /** @var \Drupal\salesforce\Rest\RestClientInterface $client */
      $client = \Drupal::service('salesforce.client');

      /** @var \Drupal\salesforce\Rest\RestResponse $response */
      $response = $client->apiCall('/services/apexrest/cpm/v2/PaymentIntent', $data, 'POST', TRUE);

      $time = round((microtime(TRUE) - $start) * 1000, 2);
      if ($event) {
        $event->addArgument('time', $time);
      }

      if ($response->getStatusCode() != 200) {
        throw new \Exception($response->getReasonPhrase(), $response->getStatusCode());
      }

      $response_data = Json::decode((string) $response->getBody());
      if ($event) {
        $event->addArgument('response_data', $response_data);
      }
      $order->setData('payment_intent_id', $response_data['Id']);

      // If there is no redirect URL then the configured payment method
      // does not need additional processing. Create the payment and continue.
      if (empty($response_data['RedirectURL'])) {

        // Save the payment and directly redirect to the next step.
        $payment->setState('authorization');
        $payment->setRemoteState('pending');
        $payment->setRemoteId($order->getData('payment_intent_id'));
        $payment->save();

        if (\Drupal::moduleHandler()->moduleExists('past')) {
          past_event_save('commerce_findock', 'installment_data', 'Installment data #' . $order->id() . '(Installment #' . $order->getData('findock_installment_id') . ')', ['data' => $response_data, 'payment' => $payment]);
        }

        /** @var \Drupal\commerce_checkout\Entity\CheckoutFlowInterface $checkout_flow */
        $checkout_flow = $order->get('checkout_flow')->entity;
        $checkout_flow_plugin = $checkout_flow->getPlugin();

        $next_step_id = $checkout_flow_plugin->getNextStepId($order->get('checkout_step')->value);
        $checkout_flow_plugin->redirectToStep($next_step_id);
      }
      else {
        $redirect_url = $response_data['RedirectURL'];
        $order->save();
      }
    }
    catch (NeedsRedirectException $e) {
      throw $e;
    }
    catch (RestException $e) {
      if ($event) {
        $event->addException($e);
        $event->addArgument('response', (string) $e->getResponse()->getBody());
      }
      throw new PaymentGatewayException(t('Failed to initialize payment: @error', ['@error' => $e->getMessage()]), $e->getCode(), $e);
    }
    catch (\Exception $e) {
      if ($event) {
        $event->addException($e);
      }
      throw new PaymentGatewayException(t('Failed to initialize payment: @error', ['@error' => $e->getMessage()]), $e->getCode(), $e);
    }
    finally {
      if ($event) {
        $event->save();
      }
    }

    if (!empty($redirect_url)) {
      $form = $this->buildRedirectForm($form, $form_state, $redirect_url, []);
    }
    return $form;
  }

}
